﻿--Vendorizer, a World of Warcraft® user interface addon.
--Copyright© 2006-2014 Connor H. Chenoweth, aka Maul - All rights reserved.

--Vendor interaction concept derived from Seerah's VendorMaster addon

--itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemID or "itemString" or "itemName" or "itemLink")

Vendorizer = {}

VendorizerSavedState = {
	sellList = {},
	saveList = {},
	buyList = {},
	itemCache = {},
}

VendorizerCharSavedState = {
	sellList = {},
	saveList = {},
	buyList = {},
	autoRepair = true,
	guildFunds = false,
}

local tgold, tsilver, tcopper, money, SD, CSD, vf, mShown, mTab = 0,0,0,0
local itemList, currList, currItem, currIndex, currBtn = {}

local tsort = table.sort

local function clearTable(table)
	if (table) then
		for k in pairs(table) do
			table[k] = nil
		end
	end
end

local function copyTable(table)

	if (table == nil) then
		return
	end

	if (type(table) ~= "table") then
		return
	end

	local data = {}

	for k, v in pairs (table) do
		if (type(v) == "table") then
			data[k] = copyTable(v)
		else
			data[k] = v
		end
	end

	return data
end

local defaultSavedState, defaultCharSavedState = copyTable(VendorizerSavedState), copyTable(VendorizerCharSavedState)

local function itemButton_OnLeave(self)
	self.nosell = nil
	MerchantFrame.selectedTab = mTab
end

local function getItemID(item)

	local	_, link = GetItemInfo(item)

	if (link) then

		_, itemID = (":"):split(link)

		if (itemID) then
			return itemID
		end
	end
end

local function itemButton_OnEnter(self)

	if (MerchantFrame:IsShown()) then

		self:HookScript("OnLeave", itemButton_OnLeave)

		mTab = MerchantFrame.selectedTab

		local link = GetContainerItemLink(self:GetParent():GetID(), self:GetID())

		if (link) then

			local name = GetItemInfo(link)

			if (name and CSD.saveList[name]) then

				MerchantFrame.selectedTab = 2

				self.nosell = true
				self.UpdateTooltip = nil

				GameTooltip:AddLine("\nDo not sell")
				GameTooltip:Show()

				ResetCursor()
			end
		end
	end
end

local function autoRepairButtonUpdate()

	MerchantRepairText:Hide()

	vf.ar:ClearAllPoints()

	if (CanGuildBankRepair()) then

		vf.ar.text:SetText("Auto "..REPAIR_ITEMS)

		local offset = vf.ar.text:GetWidth()/2

		vf.ar:SetPoint("BOTTOM", MerchantRepairAllButton, "TOP", -offset, -2)
	else

		local repair = gsub(REPAIR_ITEMS, " ", "\n")

		vf.ar.text:SetText("Auto\n"..repair)

		local offset = vf.ar.text:GetWidth()

		vf.ar:SetPoint("RIGHT", MerchantRepairItemButton, "LEFT", -offset, 0)
	end

	if (CSD.autoRepair) then

		vf.ar:SetChecked(1)
		vf.gf:Show()

		if (CSD.guildFunds) then
			vf.gf:SetChecked(1)
		else
			vf.gf:SetChecked(nil)
		end
	else
		vf.ar:SetChecked(nil)
		vf.gf:Hide()
	end
end

local function setSelection(btn, item)

	local texture = GetItemIcon(item) or SD.itemCache[item]

	currBtn = btn; currBtn.item = item

	vf.container.hf:SetPoint("TOPLEFT", btn, "TOPLEFT", 3, 0)
	vf.container.hf:SetPoint("BOTTOMRIGHT", btn, "BOTTOMRIGHT", -3, 0)
	vf.container.hf:Show()
	vf.container.allchars:Show()
	vf.container.allchars:SetChecked(nil)

	if (vf.savetab:GetChecked()) then
		vf.savetab.nt:SetTexture(texture)
		if (SD.saveList[item]) then
			vf.container.allchars:SetChecked(1)
		end
	elseif (vf.selltab:GetChecked()) then
		vf.selltab.nt:SetTexture(texture)
		if (SD.sellList[item]) then
			vf.container.allchars:SetChecked(1)
		end
	elseif (vf.buytab:GetChecked()) then
		vf.buytab.nt:SetTexture(texture)
		if (SD.buyList[item]) then
			vf.container.allchars:SetChecked(1)
		end
	end
end

local function frameUpdate()

	local offset = FauxScrollFrame_GetOffset(vf.container.scroll)

	vf.container.hf:Hide()
	vf.container.allchars:Hide()
	vf.container.adjqty.inputbox:SetText("")
	vf.container.adjqty.inputbox:ClearFocus()

	local index, btn, btnText, item = 1

	while (_G["VendorizerFrameContainerItem"..index]) do
		_G["VendorizerFrameContainerItem"..index]:Hide()
		_G["VendorizerFrameContainerItem"..index].item = nil
		index = index + 1
	end

	for i=1,6 do

		index = i + offset

		btn = _G["VendorizerFrameContainerItem"..i]
		btnText = _G["VendorizerFrameContainerItem"..i.."Text"]

		item = itemList[index]

		if (item) then

			btn.item = item

			btn:SetID(index)
			btn:Show()
			btn:SetNormalTexture("")

			btnText:SetText(item)

			if (item == currItem) then

				setSelection(btn, item)

				btn:LockHighlight()
				btn.isHighlighted = true

				if (vf.buytab:GetChecked() and CSD.buyList[item]) then
					vf.container.adjqty.inputbox:SetText(CSD.buyList[item])
				end

			else
				btn:UnlockHighlight()
				btn.isHighlighted = false
			end
		end

	end

	FauxScrollFrame_Update(vf.container.scroll, #itemList, 6, 16, nil, nil, nil, vf.container.hf, vf:GetWidth()-93, vf:GetWidth()-74)

end

local function updateSorting(table)

	clearTable(itemList)

	if (not table) then
		frameUpdate(); return
	end

	local count = 1

	for k,v in pairs(table) do
		itemList[count] = k; count = count + 1
	end

	tsort(itemList)

	frameUpdate()

end

local function updateCurrProfit(diff)

	local color = "|cffffffff"

	if (diff < 0) then
		color="|cfff00000"
	end

	diff = abs(diff)

	local gold = floor(diff/10000)
	local silver = floor((diff-(gold*10000))/100)
	local copper = mod(diff,100)

    	vf.gold:SetText(color..gold.."|r")
    	vf.silver:SetText(color..silver.."|r")
    	vf.copper:SetText(color..copper.."|r")
end

local function autoRepair()

	if (CSD.autoRepair) then

		local cost, canRepair = GetRepairAllCost()

		if (canRepair and CanMerchantRepair()) then

			if (CanGuildBankRepair() and CSD.guildFunds) then

				RepairAllItems(1)

				cost, canRepair = GetRepairAllCost()
			end

			if (cost > GetMoney()) then

				UIErrorsFrame:AddMessage("You lack the funds to fully repair automatically")
			else
				RepairAllItems()
			end
		end
	end
end

local function vendorItems()

	for bag=0,4 do

		for slot=1, GetContainerNumSlots(bag) do

			local link, name, rarity, vendorPrice, _ = GetContainerItemLink(bag,slot)

			if (link) then
				name, _, rarity, _, _, _, _, _, _, _, vendorPrice = GetItemInfo(link)
			end

			if (name) then

				if (rarity == 0 and vendorPrice > 0) then

					if (not CSD.saveList[name]) then
						UseContainerItem(bag,slot)
					end

				else

					for k,v in pairs(CSD.sellList) do

	    					if (v and name:find(k) and not CSD.saveList[k]) then

	    						UseContainerItem(bag,slot)
	    					end
	    				end
	    			end
    			end
    		end
    	end
end

local function buyItems()

	local _, qty, name, id, batch, bought, numAvailable, maxStack

	for i=1,GetMerchantNumItems() do

		name, _, _, batch, numAvailable = GetMerchantItemInfo(i)

		if (CSD.buyList[name]) then

			id = getItemID(name)

			if (id) then

				print(name..": "..GetItemCount(id))

				qty = CSD.buyList[name] - GetItemCount(id)

				if (batch > 1) then
					qty = floor(qty/batch)
				end

				if (qty >= 1) then

					_, _, _, _, _, _, _, maxStack = GetItemInfo(name)

					if (qty > maxStack) then

						local stacks, partial = floor(qty/maxStack), mod(qty,maxStack)

						for buy=1,stacks do
							BuyMerchantItem(i, maxStack)
						end

						if (partial > 0) then
							BuyMerchantItem(i, partial)
						end
					else
						BuyMerchantItem(i, qty)
					end
				end
			end
		end
	end
end

-- save tab

function Vendorizer.SaveTab_OnLoad(self)

	self:RegisterForDrag("LeftButton", "RightButton")
	self.nt = self:GetNormalTexture()
	self.ct = self:GetCheckedTexture()
	self:GetParent().savetab = self

end

function Vendorizer.SaveTab_OnClick(self, drag)

	PlaySound("igCharacterInfoTab")

	vf.selltab:SetChecked(nil)
	vf.buytab:SetChecked(nil)
	vf.container.adjqty:Hide()
	vf.container.header:SetText("Save List")

	if (self:GetChecked()) then

		if (not drag) then
			vf.open = true; vf.close = false
		end

		updateSorting(CSD.saveList)
	else
		vf.close = true; vf.open = false
	end

end

function Vendorizer.SaveTab_OnShow(self)

	self.nt:SetTexture("Interface\\Icons\\Spell_Nature_MoonKey")
	self:SetChecked(nil)

end

function Vendorizer.SaveTab_OnReceiveDrag(self)

	local cursorType, action1 = GetCursorInfo()

	if (cursorType == "item" or  cursorType == "merchant") then

		local name, texture

		if (cursorType == "item") then

			name, _, _, _, _, _, _, _, _, texture = GetItemInfo(action1)

		elseif (cursorType == "merchant") then

			name, texture = GetMerchantItemInfo(action1)

		end

		CSD.saveList[name] = true
		SD.itemCache[name] = texture

		self:SetChecked(1)
		self.nt:SetTexture(texture)

		currItem = name

		Vendorizer.SaveTab_OnClick(self, drag)

		ClearCursor(); SetCursor(nil)

		updateSorting(CSD.saveList)

	end

end

-- sell tab

function Vendorizer.SellTab_OnLoad(self)

	self:RegisterForDrag("LeftButton", "RightButton")
	self.nt = self:GetNormalTexture()
	self.ct = self:GetCheckedTexture()
	self:GetParent().selltab = self

end

function Vendorizer.SellTab_OnClick(self, drag)

	PlaySound("igCharacterInfoTab")

	vf.savetab:SetChecked(nil)
	vf.buytab:SetChecked(nil)
	vf.container.adjqty:Hide()
	vf.container.header:SetText("Sell List")

	if (self:GetChecked()) then
		if (not drag) then
			vf.open = true; vf.close = false
		end
		updateSorting(CSD.sellList)
	else
		vf.close = true; vf.open = false
	end
end

function Vendorizer.SellTab_OnShow(self)

	self.nt:SetTexture("Interface\\Icons\\Spell_ChargeNegative")
	self:SetChecked(nil)

end

function Vendorizer.SellTab_OnReceiveDrag(self)

	local cursorType, action1, action2 = GetCursorInfo()

	if (cursorType == "item") then

		local name, _, _, _, _, _, _, _, _, texture, price = GetItemInfo(action1)

		if (price > 0) then

			CSD.sellList[name] = true
			SD.itemCache[name] = texture

			self:SetChecked(1)
			self.nt:SetTexture(texture)

			currItem = name

			Vendorizer.SellTab_OnClick(self, true)

			vendorItems()

		else
			UIErrorsFrame:AddMessage(ERR_VENDOR_NOT_INTERESTED, 1.0, 0.1, 0.1, 1.0)
		end

		ClearCursor(); SetCursor(nil)

		updateSorting(CSD.sellList)
	end
end

-- buy tab

function Vendorizer.BuyTab_OnLoad(self)

	self:RegisterForDrag("LeftButton", "RightButton")
	self.nt = self:GetNormalTexture()
	self.ct = self:GetCheckedTexture()
	self:GetParent().buytab = self

end

function Vendorizer.BuyTab_OnClick(self, drag)

	local cursorType, action1 = GetCursorInfo()

	if (cursorType and not drag) then
		Vendorizer.BuyTab_OnReceiveDrag(self)
	end

	PlaySound("igCharacterInfoTab")

	vf.savetab:SetChecked(nil)
	vf.selltab:SetChecked(nil)
	vf.container.adjqty:Show()
	vf.container.header:SetText("Buy List")

	if (self:GetChecked()) then
		vf.open = true; vf.close = false
		updateSorting(CSD.buyList)
	else
		vf.close = true; vf.open = false
	end

end

function Vendorizer.BuyTab_OnShow(self)

	self.nt:SetTexture("Interface\\Icons\\Spell_ChargePositive")
	self:SetChecked(nil)

end

function Vendorizer.BuyTab_OnReceiveDrag(self, click)

	local cursorType, action1 = GetCursorInfo()

	if (cursorType == "merchant") then

		local name, texture = GetMerchantItemInfo(action1)

		CSD.buyList[name] = 1
		SD.itemCache[name] = texture

		self:SetChecked(1)
		self.nt:SetTexture(texture)

		currItem = name

		if (not click) then
			Vendorizer.BuyTab_OnClick(self, true)
		end

		vf.container.adjqty.inputbox:SetFocus()

		buyItems()

		ClearCursor(); SetCursor(nil)

	elseif (cursorType == "item") then

		local name, _, _, _, _, _, _, _, _, texture = GetItemInfo(action1)

		CSD.buyList[name] = 1
		SD.itemCache[name] = texture

		self:SetChecked(1)
		self.nt:SetTexture(texture)

		currItem = name

		if (not click) then
			Vendorizer.BuyTab_OnClick(self, true)
		end

		vf.container.adjqty.inputbox:SetFocus()

		buyItems()

		ClearCursor(); SetCursor(nil)
	end
end

function Vendorizer.Frame_OnShow(self)

	vf.container.header:SetText("")

	tgold, tsilver, tcopper = 0, 0, 0

	updateSorting()

end

function Vendorizer.Frame_OnHide(self)

	self.width = 1
	self:SetWidth(1)
	self:SetBackdropBorderColor(0.5, 0.5, 0.5, 0)

	vf.container:Hide()

end

function Vendorizer.FrameContainer_Reset(self)

	Vendorizer.SaveTab_OnShow(vf.savetab)
	Vendorizer.SellTab_OnShow(vf.selltab)
	Vendorizer.BuyTab_OnShow(vf.buytab)
end

function Vendorizer.Frame_OnUpdate(self, elasped)

	if (self.open) then

		if (self.width > 34) then
			self:SetBackdropBorderColor(0.7, 0.7, 0.7, 1)
		end

		self.width = self.width + 20

		if (self.width > 225) then

			self.width = 225
			self.open = false
			self:SetBackdropBorderColor(0.7, 0.7, 0.7, 1)

			vf.container:Show()
		end

		self:SetWidth(self.width)
	end

	if (self.close) then

		if (vf.container:IsVisible()) then
			vf.container:Hide()
			vf.container.hf:Hide()
		end

		self.width = self.width - 20

		if (self.width < 35) then
			self:SetBackdropBorderColor(0.7, 0.7, 0.7, 0)
		end

		if (self.width < 2) then
			self.width = 2
			self.close = false
			self:SetBackdropBorderColor(0.7, 0.7, 0.7, 0)
		end

		self:SetWidth(self.width)
	end
end

function Vendorizer.AutoRepair_OnClick(self)

	if (self:GetChecked()) then
		CSD.autoRepair = true; autoRepair(); vf.gf:Show()
	else
		CSD.autoRepair = false; vf.gf:Hide()
	end
end

function Vendorizer.GuildFunds_OnClick(self)

	if (self:GetChecked()) then
		CSD.guildFunds = true
	else
		CSD.guildFunds = false
	end

end

function Vendorizer.ScrollFrame_OnVerticalScroll(self, offset)

	if (not offset) then
		offset = FauxScrollFrame_GetOffset(self)
	end

	FauxScrollFrame_OnVerticalScroll(self, offset, 16, frameUpdate)
end

function Vendorizer.Remove_OnClick(self)

	if (currBtn and currBtn.item) then

		if (vf.buytab:GetChecked()) then

			CSD.buyList[currBtn.item] = nil

			vf.buytab.nt:SetTexture("Interface\\Icons\\Spell_ChargePositive")

			updateSorting(CSD.buyList)

		elseif (vf.selltab:GetChecked()) then

			CSD.sellList[currBtn.item] = nil

			vf.selltab.nt:SetTexture("Interface\\Icons\\Spell_ChargeNegative")

			updateSorting(CSD.sellList)

		elseif (vf.savetab:GetChecked()) then

			CSD.saveList[currBtn.item] = nil

			vf.savetab.nt:SetTexture("Interface\\Icons\\Spell_Nature_MoonKey")

			updateSorting(CSD.saveList)
		end
	end
end

function Vendorizer.AdjQtyFrame_OnClick(add, edit)

	if (vf.buytab:GetChecked() and currBtn and currBtn.item and CSD.buyList[currBtn.item]) then

		local qty = CSD.buyList[currBtn.item]

		if (edit) then

			qty = tonumber(vf.container.adjqty.inputbox:GetText())

			if (not qty) then
				qty = 1
			end

		elseif (add) then

			qty = qty + 1

		else
			qty = qty - 1

			if (qty < 1) then
				qty = 1
			end
		end

		CSD.buyList[currBtn.item] = qty

		vf.container.adjqty.inputbox:SetText(CSD.buyList[currBtn.item])

		buyItems()
	end
end

function Vendorizer.ItemButton_OnClick(self, button)

	if (self.item) then

		setSelection(self, self.item)

		if (vf.buytab:GetChecked() and CSD.buyList[self.item]) then
			vf.container.adjqty.inputbox:SetText(CSD.buyList[self.item])
		end
	end

end

function Vendorizer.AllChars_OnClick(self)

	if (currBtn and currBtn.item) then

		if (vf.buytab:GetChecked()) then

			if (self:GetChecked()) then
				SD.buyList[currBtn.item] = true
			else
				SD.buyList[currBtn.item] = nil
			end

		elseif (vf.selltab:GetChecked()) then

			if (self:GetChecked()) then
				SD.sellList[currBtn.item] = true
			else
				SD.sellList[currBtn.item] = nil
			end

		elseif (vf.savetab:GetChecked()) then

			if (self:GetChecked()) then
				SD.saveList[currBtn.item] = true
			else
				SD.saveList[currBtn.item] = nil
			end
		end
	end
end

local function controlOnEvent(self, event, ...)

	if (event == "ADDON_LOADED" and ... == "Vendorizer") then

		SD = VendorizerSavedState; CSD = VendorizerCharSavedState

		vf = VendorizerFrame
		vf.width = 1

		for k,v in pairs(defaultSavedState) do
			if (SD[k] == nil) then
				SD[k] = v
			end
		end

		for k,v in pairs(defaultCharSavedState) do
			if (CSD[k] == nil) then
				CSD[k] = v
			end
		end

		for k,v in pairs(SD.saveList) do
			if (v) then
				CSD.saveList[k] = true
			end
		end

		for k,v in pairs(SD.sellList) do
			if (v) then
				CSD.sellList[k] = true
			end
		end

		for k,v in pairs(SD.buyList) do
			if (v) then
				CSD.buyList[k] = true
			end
		end

		hooksecurefunc("ContainerFrameItemButton_OnEnter", itemButton_OnEnter)
		hooksecurefunc("MerchantFrame_UpdateRepairButtons", autoRepairButtonUpdate)

	elseif (event == "MERCHANT_SHOW") then

		mShown = true

		money = GetMoney()

		updateCurrProfit(0)

		vendorItems(); buyItems(); autoRepair()

	elseif (event == "MERCHANT_CLOSED") then

		mShown = false

	elseif ((event == "PLAYER_MONEY" or
	         event == "GUILDBANK_UPDATE_MONEY" or
	         event == "GUILDBANK_UPDATE_WITHDRAWMONEY")
	         and mShown) then

		updateCurrProfit(GetMoney()-money)
	end
end

local frame = CreateFrame("Frame", nil, UIParent)
frame:SetScript("OnEvent", controlOnEvent)
frame:RegisterEvent("ADDON_LOADED")
frame:RegisterEvent("MERCHANT_SHOW")
frame:RegisterEvent("MERCHANT_UPDATE")
frame:RegisterEvent("MERCHANT_CLOSED")
frame:RegisterEvent("BAG_UPDATE")
frame:RegisterEvent("PLAYER_MONEY")
frame:RegisterEvent("GUILDBANK_UPDATE_MONEY")
frame:RegisterEvent("GUILDBANK_UPDATE_WITHDRAWMONEY")
